@@ -1,6 +1,8 @@
 #include <libavformat/avformat.h>
 #include <libavfilter/buffersink.h>
 #include <libavfilter/buffersrc.h>
+#include <libavcodec/avcodec.h>
+#include <libavutil/channel_layout.h>
 #include <libavutil/opt.h>
 
 #define R_NO_REMAP
@@ -9,6 +11,10 @@
 #define VIDEO_TIME_BASE 1000
 #include <Rinternals.h>
 
+#if LIBAVCODEC_VERSION_MAJOR >= 59
+#define NEW_CHANNEL_API
+#endif
+
 int total_open_handles = 0;
 
 typedef struct {
@@ -25,7 +31,7 @@ typedef struct {
 } filter_container;
 
 typedef struct {
-  AVCodec *codec;
+  const AVCodec *codec;
   AVFormatContext *muxer;
   input_container *audio_input;
   input_container *video_input;
@@ -57,7 +63,7 @@ static void bail_if(int ret, const char * what){
     Rf_errorcall(R_NilValue, "FFMPEG error in '%s': %s", what, av_err2str(ret));
 }
 
-static void bail_if_null(void * ptr, const char * what){
+static void bail_if_null(const void * ptr, const char * what){
   if(!ptr)
     bail_if(-1, what);
 }
@@ -165,13 +171,18 @@ static input_container *open_audio_input(const char *filename){
   /* Try all input streams */
   int si = find_stream_audio(demuxer, filename);
   AVStream *stream = demuxer->streams[si];
-  AVCodec *codec = avcodec_find_decoder(stream->codecpar->codec_id);
+  const AVCodec *codec = avcodec_find_decoder(stream->codecpar->codec_id);
   bail_if_null(codec, "avcodec_find_decoder");
   AVCodecContext *decoder = avcodec_alloc_context3(codec);
   bail_if(avcodec_parameters_to_context(decoder, stream->codecpar), "avcodec_parameters_to_context");
   bail_if(avcodec_open2(decoder, codec, NULL), "avcodec_open2 (audio)");
-  if (!decoder->channel_layout) /* Is this needed ?*/
+#ifdef NEW_CHANNEL_API
+  if (decoder->ch_layout.order == AV_CHANNEL_ORDER_UNSPEC)
+    av_channel_layout_default(&decoder->ch_layout, decoder->ch_layout.nb_channels);
+#else
+  if(!decoder->channel_layout)
     decoder->channel_layout = av_get_default_channel_layout(decoder->channels);
+#endif
   return new_input_container(demuxer, decoder, demuxer->streams[si]);
 }
 
@@ -179,14 +190,23 @@ static filter_container *open_audio_filter(AVCodecContext *decoder, AVCodecConte
 
   /* Create a new filter graph */
   AVFilterGraph *filter_graph = avfilter_graph_alloc();
-
   char input_args[512];
+
+#ifdef NEW_CHANNEL_API
+  char buf[64] = {0};
+  av_channel_layout_describe(&decoder->ch_layout, buf, sizeof(buf));
+  snprintf(input_args, sizeof(input_args),
+           "time_base=%d/%d:sample_rate=%d:sample_fmt=%s:channel_layout=%s",
+           decoder->time_base.num, decoder->time_base.den, decoder->sample_rate,
+           av_get_sample_fmt_name(decoder->sample_fmt),
+           buf);
+#else
   snprintf(input_args, sizeof(input_args),
            "time_base=%d/%d:sample_rate=%d:sample_fmt=%s:channel_layout=0x%" PRIx64,
            decoder->time_base.num, decoder->time_base.den, decoder->sample_rate,
            av_get_sample_fmt_name(decoder->sample_fmt),
            decoder->channel_layout);
-
+#endif
   AVFilterContext *buffersrc_ctx = NULL;
   bail_if(avfilter_graph_create_filter(&buffersrc_ctx, avfilter_get_by_name("abuffer"), "audiosrc",
                                        input_args, NULL, filter_graph), "avfilter_graph_create_filter (audio/src)");
@@ -200,9 +220,15 @@ static filter_container *open_audio_filter(AVCodecContext *decoder, AVCodecConte
   bail_if(av_opt_set_bin(buffersink_ctx, "sample_fmts",
                          (uint8_t*)&encoder->sample_fmt, sizeof(encoder->sample_fmt),
                          AV_OPT_SEARCH_CHILDREN), "av_opt_set_bin (sample_fmts)");
+#ifdef NEW_CHANNEL_API
+  av_channel_layout_describe(&encoder->ch_layout, buf, sizeof(buf));
+  bail_if(av_opt_set(buffersink_ctx, "ch_layouts",
+                   buf, AV_OPT_SEARCH_CHILDREN), "av_opt_set (ch_layouts)");
+#else
   bail_if(av_opt_set_bin(buffersink_ctx, "channel_layouts",
                          (uint8_t*)&encoder->channel_layout, sizeof(encoder->channel_layout),
                          AV_OPT_SEARCH_CHILDREN), "av_opt_set_bin (channel_layouts)");
+#endif
   bail_if(av_opt_set_bin(buffersink_ctx, "sample_rates",
                          (uint8_t*)&encoder->sample_rate, sizeof(encoder->sample_rate),
                          AV_OPT_SEARCH_CHILDREN), "av_opt_set_bin (sample_rates)");
@@ -312,12 +338,16 @@ static void add_video_output(output_container *output, int width, int height){
 
 static void add_audio_output(output_container *container){
   AVCodecContext *audio_decoder = container->audio_input->decoder;
-  AVCodec *output_codec = avcodec_find_encoder(container->muxer->oformat->audio_codec);
+  const AVCodec *output_codec = avcodec_find_encoder(container->muxer->oformat->audio_codec);
   bail_if_null(output_codec, "Failed to find default audio codec");
   AVCodecContext *audio_encoder = avcodec_alloc_context3(output_codec);
   bail_if_null(audio_encoder, "avcodec_alloc_context3 (audio)");
+#ifdef NEW_CHANNEL_API
+  av_channel_layout_default(&audio_encoder->ch_layout, container->channels ? container->channels : audio_decoder->ch_layout.nb_channels);
+#else
   audio_encoder->channels = container->channels ? container->channels : audio_decoder->channels;
   audio_encoder->channel_layout = av_get_default_channel_layout(audio_encoder->channels);
+#endif
   audio_encoder->sample_rate = container->sample_rate ? container->sample_rate : audio_decoder->sample_rate;
   audio_encoder->sample_fmt = output_codec->sample_fmts[0];
   audio_encoder->strict_std_compliance = FF_COMPLIANCE_EXPERIMENTAL;
@@ -371,7 +401,7 @@ static void sync_audio_stream(output_container * output, int64_t pts){
     frame = av_frame_alloc();
   }
   while(force_everything || force_flush ||
-        av_compare_ts(audio_stream->cur_dts, audio_stream->time_base,
+        av_compare_ts(av_stream_get_end_pts(audio_stream), audio_stream->time_base,
                                      pts, output->video_stream->time_base) < 0) {
     int ret = avcodec_receive_packet(output->audio_encoder, pkt);
     if (ret == AVERROR(EAGAIN)){
@@ -522,7 +552,7 @@ static void read_from_input(const char *filename, output_container *output){
   bail_if(avformat_find_stream_info(demuxer, NULL), "avformat_find_stream_info");
   int si = find_stream_video(demuxer, filename);
   AVStream *stream = demuxer->streams[si];
-  AVCodec *codec = avcodec_find_decoder(stream->codecpar->codec_id);
+  const AVCodec *codec = avcodec_find_decoder(stream->codecpar->codec_id);
   bail_if_null(codec, "avcodec_find_decoder");
   AVCodecContext *decoder = avcodec_alloc_context3(codec);
 
@@ -586,15 +616,15 @@ static SEXP encode_input_files(void *ptr){
   return R_NilValue;
 }
 
-static AVCodec *get_default_codec(const char *filename){
-  AVOutputFormat *frmt = av_guess_format(NULL, filename, NULL);
+static const AVCodec *get_default_codec(const char *filename){
+  const AVOutputFormat *frmt = av_guess_format(NULL, filename, NULL);
   bail_if_null(frmt, "av_guess_format");
   return avcodec_find_encoder(frmt->video_codec);
 }
 
 SEXP R_encode_video(SEXP in_files, SEXP out_file, SEXP framerate, SEXP vfilter,
                     SEXP enc, SEXP audio){
-  AVCodec *codec = Rf_length(enc) ?
+  const AVCodec *codec = Rf_length(enc) ?
     avcodec_find_encoder_by_name(CHAR(STRING_ELT(enc, 0))) :
     get_default_codec(CHAR(STRING_ELT(out_file, 0)));
   bail_if_null(codec, "avcodec_find_encoder_by_name");
